# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from wtforms import Form, BooleanField, TextField, TextAreaField, validators
from flask import request, session, render_template, redirect, g, url_for, \
                  flash, abort

from . import app, login_required
from ..model import User, Realm
from .. import model


class RealmForm(Form):
    name = TextField('Name', [validators.Length(min=4, max=25)])
    description = TextAreaField('Description', [validators.Length(min=6)])


class AddUserForm(Form):
    nick = TextField('Nick', [validators.Required()])

    def validate_nick(form, field):
        if not User.find_by_nick(field.data):
            raise validators.ValidationError("unknown Nickname")


@app.route("/")
def index():
    return render_template("index.html.jinja", realms=Realm.find_all())


@app.route("/realms/new/", methods=["GET", "POST"])
@login_required
def create_realm():
    if request.method == 'POST':

        form = RealmForm(request.form)

        if form.validate():
            realm = Realm()
            realm.owners = [g.user.openid]
            realm.name = request.form["name"]
            realm.description = request.form["description"]
            realm.save()

            flash("Realm successfully saved")
            return redirect(url_for("realm_details",
                                    realmname=request.form["name"]))
    else:
        form = RealmForm()

    return render_template('realm/create.html.jinja', form=form)


@app.route("/realms/<string:realmname>/edit/", methods=["GET", "POST"])
@login_required
def edit_realm(realmname):
    realm = Realm.find_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.method == "POST":
        form = RealmForm(request.form)
        if form.validate():
            form.populate_obj(realm)
            realm.save()
            flash("Realm successfully saved")
            return redirect(url_for("realm_details", realmname=realm.name))
    else:
        form = RealmForm(obj=realm)

    return render_template('realm/edit.html.jinja', realm=realm, form=form)


@app.route("/realms/<string:realmname>/remove/",
                                             methods=["GET", "POST", "DELETE"])
@login_required
def remove_realm(realmname):
    realm = Realm.find_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.form.get("_method", request.method) == "DELETE":
        realm.remove()
        flash("Realm successfully deleted")
        return redirect('/')

    return render_template('realm/remove.html.jinja', realm=realm)


@app.route("/realms/<string:realmname>/access", methods=["GET", "POST"])
@login_required
def set_realm_access(realmname):
    realm = Realm.find_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.method == 'POST':
        form = AddUserForm(request.form)
        if form.validate():
            realm.add_owner(User.find_by_nick(form.data["nick"]))
            realm.save()

            return redirect(url_for('set_realm_access', realmname=realmname))
    else:
        form = AddUserForm()

    return render_template('realm/access_control.html.jinja',
                           realm=realm,
                           owners=realm.get_owners(),
                           form=form)


@app.route('/realms/<string:realmname>/quit_ownership', methods=["POST"])
@login_required
def quit_realm_ownership(realmname):
    realm = Realm.find_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    realm.remove_owner(g.user)
    return redirect(url_for('realm_details', realmname=realmname))


@app.route("/realms/<string:realmname>/")
def realm_details(realmname):
    realm = Realm.find_by_name(realmname)

    return render_template('realm/show.html.jinja',
                           realm=realm,
                           owners=realm.get_owners(),
                           editable=realm.is_editable_by(g.user),
                           is_member=g.user and g.user.is_realm_member(realm),
                           users=realm.get_users())


@app.route("/profile/enter_realm/", methods=["POST"])
@login_required
def enter_realm():
    realm = Realm.find_by_name(request.form['realmname'])
    g.user.join_realm(realm)
    flash("You joined %s. Good luck!" % realm.name)
    return redirect(url_for("show_profile"))


@app.route("/realms/<string:realmname>/users/<string:username>/")
def show_realm_user(realmname, username):
    realm = Realm.find_by_name(realmname)
    user = User.find_by_nick(username)
    membership = user.realms[realmname]
    achievements = realm.doc["achievements"]
    grantable = realm.is_editable_by(g.user)

    return render_template('realm_user.html.jinja', realm=realm,
                                                    membership=membership,
                                                    grantable=grantable,
                                                    achievements=achievements)
