# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from wtforms import Form, HiddenField, TextField, validators
from flaskext.openid import OpenID
from flask import request, session, render_template, redirect, g, url_for, \
                  abort

from . import app, login_required
from ..model import User, Realm


oid = OpenID(app, '/tmp/openid')


@app.route("/users/login/", methods=["GET", "POST"])
@oid.loginhandler
def login():
    if g.user is not None:
        return redirect(oid.get_next_url())

    if request.method == 'POST':
        openid = request.form.get('openid')
        if openid:
            return oid.try_login(openid, ask_for=['nickname'])

    return render_template('user/login.html.jinja', next=oid.get_next_url(),
                                                    error=oid.fetch_error())


@oid.after_login
def create_or_login(resp):
    user = User.find_by_openid(resp.identity_url)

    if user is None:
        session['openid_candidate'] = resp.identity_url
        return redirect(url_for('register', nick=resp.nickname,
                                            next=oid.get_next_url() or ""))
    else:
        session['openid'] = resp.identity_url

    return redirect(oid.get_next_url())


@app.route("/users/logout/", methods=["GET", "POST"])
def logout():
    if request.method == 'POST':
        session['openid'] = None
        return redirect('/')
    else:
        return render_template('user/logout.html.jinja')


class RegisterForm(Form):
    nick = TextField('nick', [validators.Length(min=3, max=25)])
    next = HiddenField()

    def validate_nick(form, field):
        """Check uniqueness of username in db.

        Only for usability, actual racing conditions have to be avoided via
        database index
        """
        if User.find_by_nick(field.data):
            raise validators.ValidationError("Nickname already taken")


@app.route("/users/register/", methods=["GET", "POST"])
def register():
    if not 'openid_candidate' in session:
        abort(500)

    if User.find_by_openid(session["openid_candidate"]):
        abort(500)  # There's already a registered user with this OpenID

    if request.method == 'POST':
        form = RegisterForm(request.form)

        if form.validate():
            session["openid"] = session.pop("openid_candidate")
            user = User()
            user.openid = session["openid"]
            user.nick = form.data["nick"]
            user.save()
            return redirect(form.data["next"])
    else:
        form = RegisterForm(request.args)
        form.validate()

    return render_template('user/register.html.jinja', form=form)


@app.route("/profile/")
def show_profile():
    if not g.user:
        return redirect(url_for("login"))
    else:
        return redirect(url_for("show_user_profile", nick=g.user.nick))


@app.route("/profile/<string:nick>/")
def show_user_profile(nick):
    user = User.find_by_nick(nick)
    return render_template("user/profile.html.jinja",
                           user=user,
                           realms=user.get_realms())


@app.route("/notifications/<string:notification_id>/",
                                             methods=["GET", "POST", "DELETE"])
@login_required
def notification(notification_id):
    notification = g.user.get_notification(notification_id)

    if request.form.get("_method", request.method) == "DELETE":
        notification.remove()
        g.user.save()
        return 'done!'
    else:
        return "here_it_is"
