# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

import bson

from .base import Document, docproperty, doclistproperty
from .achievement import Achievement


class Realm(Document):
    """
    Example document:

    {
        'owners': ['someone.example.com'],
        'name': 'Scrapmetal',
        'description': 'This is the Realm dedicated to Scrapmetal itself',
        'achievements': [
            {
                'id': ObjectId('C0FFEEBABE'),
                'name': 'Basic Achievement',
                'description': 'This is a basic Achievement, given to everyone'
            },
            # ...
        ]
    }
    """
    collection_name = 'realms'

    name = docproperty("name")
    description = docproperty("description")
    owners = docproperty("owners")

    #achievements = docproperty("achievements")  # TODO a little more wrapping?
    achievements = doclistproperty("achievements", Achievement)

    @classmethod
    def empty_doc(cls):
        return {
            'owners': [],
            'name': '',
            'description': '',
            'achievements': [],
            'webhooks': {
                'incoming': []
            }
        }

    @classmethod
    def find_by_name(cls, name):
        return cls.find_one_by_filter({"name": name})

    @classmethod
    def find_all(cls):
        return cls.find_by_filter({})

    def is_editable_by(self, user):
        return user is not None and user.openid in self.owners

    def add_owner(self, user):
        if user.openid in self.owners:
            return

        self.owners.append(user.openid)
        self.save()

        for owner in self.get_owners():
            owner.notify("%s is now in control of %s, too." % (user.nick,
                                                               self.name))
            owner.save()

    def remove_owner(self, user):
        if len(self.owners) == 1:
            raise RuntimeError("last user cannot leave")
        self.owners.remove(user.openid)
        self.save()

        for owner in self.get_owners():
            owner.notify("%s gave up ownership of %s" % (user.nick, self.name))
            owner.save()

    def add_achievement(self, name, description):
        self.doc["achievements"].append({
            'id': bson.ObjectId(),
            'name': name,
            'description': description,
        })
        self.save()

    def remove_achievement(self, name):
        self.doc["achievements"] = [a for a in self.doc["achievements"]
                                        if a["name"] != name]
        self.save()

    def get_owners(self):
        from .user import User
        return User.find_by_filter({'openid': {'$in': self.owners}})

    def get_achievement_by_name(self, achievementname):
        return [achievement for achievement in self.achievements
                            if achievement.name == achievementname][0]

    def get_granted_achievements(self, user):
        granted_ids = user.doc["realms"][self.name]["achievements"]
        return [achievement for achievement in self.doc["achievements"]
                            if achievement["id"] in granted_ids]

    def get_completion(self, user):
        granted = len(self.get_granted_achievements(user))
        total = len(self.achievements)
        return granted, total, (1.0 * granted / total if total > 0 else 0)

    def get_users(self):
        from .user import User
        return User.find_by_filter({
                                   'realms.%s' % self.name: {'$exists': True}})
