# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

import bson

from .base import Document, docproperty


class User(Document):
    """
    Example document:

    {
        'openid': 'someone.example.com',
        'nick': 'someone',
        'realms': {
            'Scrapmetal': {
                'name': 'keppla',
                'achievements': ["Basic Achievement", ...]
            }
        },
        'notifications': [
            # see `Notification`
        ],
        'is_admin': False
    }
    """
    collection_name = 'users'

    nick = docproperty("nick")
    openid = docproperty("openid")
    realms = docproperty("realms")
    is_admin = docproperty("is_admin")

    @property
    def notifications(self):
        from .notification import Notification
        return [Notification(self, subdoc)
                                       for subdoc in self.doc['notifications']]

    @classmethod
    def empty_doc(cls):
        return {
            'openid': '',
            'nick': '',
            'realms': {},
            'notifications': [],
            'is_admin': False,
        }

    @classmethod
    def find_by_openid(cls, openid):
        return cls.find_one_by_filter({"openid": openid})

    @classmethod
    def find_by_nick(cls, nick):
        return cls.find_one_by_filter({"nick": nick})

    def grant_achievement(self, achievement):
        self.realms[achievement.realm.name]["achievements"].append(
                                                                achievement.id)
        self.notify('You gained an achievement!')

    def revoke_achievement(self, achievement):
        self.realms[achievement.realm.name]["achievements"].remove(
                                                                achievement.id)
        self.notify('One of your achievements was revoked!')

    def has_achievement(self, achievement):
        return any(achievement.id in membership['achievements']
                                        for membership in self.realms.values())

    def get_realms(self):
        from .realm import Realm
        return Realm.find_by_filter({"name": {"$in": self.realms.keys()}})

    def get_notification(self, notification_id):
        idobj = bson.ObjectId(notification_id)
        return [n for n in self.notifications if n.id == idobj][0]

    def notify(self, text):
        from .notification import Notification
        Notification.create(self, text)

    def is_realm_member(self, realm):
        return realm.name in self.realms

    def join_realm(self, realm):
        if realm.name in self.realms:
            raise RuntimeError("already in realm")

        self.realms[realm.name] = {
            'name': self.nick,
            'achievements': [],
        }
        self.save()
